@@ -8,20 +8,16 @@ import datetime
 from dateutil.relativedelta import relativedelta, FR
 from bs4 import BeautifulSoup
 
-
 class Database:
 
     def __init__(self,authToken):
         self.authToken = authToken
 
     def get_hk_stock_ohlc(self, code, start_date, end_date, freq, price_adj=False, vol_adj=False):
-        # http://localhost:8000/api/cbbc_dis?sid=0910d3e18c01f86&token=efc220c6c6da52b&date=20210802&database=cbbc_dis
-        # http://localhost:8000/data_api?sid=0910d3e18c01f86&token=efc220c6c6da52b&date=20210802&database=cbbc_dis
 
         check_bool_dict = self.check_hk_stock_ohlc_args(code, start_date, end_date, freq)
 
         if False not in list(check_bool_dict.values()):
-            link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'hk_stock_ohlc'
@@ -29,45 +25,91 @@ class Database:
             start_date_str = 'start_date=' + str(start_date)
             end_date_str = 'end_date=' + str(end_date)
             freq_str = 'freq=' + freq
-            price_adj = 'price_adj=0' if price_adj == False else 'price_adj=1'
-            vol_adj = 'vol_adj=0' if vol_adj == False else 'vol_adj=1'
+            price_adj_str = 'price_adj=0' if price_adj == False else 'price_adj=1'
+            vol_adj_str = 'vol_adj=0' if vol_adj == False else 'vol_adj=1'
             link_str = link_url + code_str + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + \
-                       end_date_str + '&' + freq_str + '&' + price_adj + '&' + vol_adj
-
-            response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
-
-            df = pd.DataFrame(result)
-            cols = ['datetime'] + list(df.columns)
+                       end_date_str + '&' + freq_str + '&' + price_adj_str + '&' + vol_adj_str
             
-            if 'T' in freq:
-                df['time'] = df['time'].astype(str)
-                df['datetime'] = df['date'] + ' ' + df['time']
-                df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+            response = requests.get(link_str)
+            
+            if price_adj == False:
+                ohlc_result = json.loads(json.loads(response.content).replace("'", "\""))
             else:
-                df['datetime'] = df['date']
-                df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
+                result = ast.literal_eval(json.loads(response.content))
+                ohlc_result = result[0]
+                qfq_result  = result[1]
+                qfq_df = pd.DataFrame(qfq_result, columns=['date', 'qfq_factor'])
 
-            df = df[cols]
-            df = df.set_index(keys='datetime')
-            df = df[['open', 'high', 'low', 'close', 'volume']]
-            df = df.sort_index()
+            df_list = []
+
+            for item in ohlc_result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                cols = ['datetime', 'date'] + list(df.columns)
+
+                if len(df) > 0:
+                    if 'T' in freq:
+                        df['time'] = df['time'].astype(str)
+                        df['date'] = str(date_int)
+                        df['datetime'] = df['date'] + ' ' + df['time']
+                        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+
+                    else:
+                        df['date'] = str(date_int)
+                        df['datetime'] = df['date']
+                        df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
+
+                    df = df[cols]
+                    df = df.set_index(keys='datetime')
+                    df_list.append(df)
+                    
+            df = pd.concat(df_list)
+            
+            if price_adj == True:
+                qfq_df = qfq_df[qfq_df['date'] > 20100000]
+                qfq_df = qfq_df.iloc[::-1].reset_index(drop=True)
+                qfq_df['date'] = qfq_df['date'].astype(str)
+                first_date = qfq_df.loc[qfq_df.index[0], 'date'] 
+                last_date = str(max([int(qfq_df.loc[qfq_df.index[-1], 'date']), int(df.loc[df.index[-1], 'date'])]))
+                qfq_df['date'] = pd.to_datetime(qfq_df['date'], format='%Y-%m-%d')
+                qfq_df = qfq_df.set_index(keys='date')
+                t_index = pd.DatetimeIndex(pd.date_range(start=first_date, end=last_date, freq='1D'))
+                qfq_df = qfq_df.reindex(t_index)
+                qfq_df = qfq_df.fillna(method='ffill')
+                qfq_df = qfq_df.reset_index()
+                qfq_df['index'] = qfq_df['index'].astype(str)
+                qfq_df['index'] = qfq_df['index'].str.replace('-', '')
+                df = pd.merge(left=df, left_on='date', right=qfq_df, right_on='index')
+
+                if 'T' in freq:
+                    df['datetime'] = df['date'] + ' ' + df['time']
+                    df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+                else:
+                    df['datetime'] = df['date']
+                    df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
+
+                if vol_adj == True:
+                    col_list = ['open', 'high', 'low', 'close']
+                else:
+                    col_list = ['open', 'high', 'low', 'close', 'volume']
+
+                for col in col_list:
+                    df[col] = df[col] * df['qfq_factor']
+                    if col != 'volume':
+                        df[col] = df[col].map(lambda x: round(x, 3))
+                    else:
+                        df[col] = df[col].map(lambda x: round(x, 0))
+                        df[col] = df[col].astype(int)
+
+                df = df[['date', 'time', 'open', 'high', 'low', 'close', 'volume']]
 
             return df
-
-        else:
-            err_msg = 'Error in: '
-            for error in check_bool_dict:
-                if check_bool_dict[error] == False:
-                    err_msg += error + ','
-            print(err_msg)
     
     def get_us_stock_ohlc(self, code, start_date, end_date):
 
         check_bool_dict = self.check_us_stock_ohlc_args(code, start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            # link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'spx_stock_ohlc'
@@ -78,21 +120,28 @@ class Database:
                        end_date_str
 
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            #result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    cols = ['datetime', 'date'] + list(df.columns)
+                    df['time'] = df['time'].astype(str)
+                    df['date'] = str(date_int)
+                    df['datetime'] = df['date'] + ' ' + df['time'].str.zfill(6)
+                    df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
 
-            cols = ['datetime'] + list(df.columns)
-            df['time'] = df['time'].astype(str)
-            df['time'] = df['time'].str.zfill(6)
-            df['datetime'] = df['date'] + ' ' + df['time']
-            df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
+                    df = df[cols]
+                    df = df.set_index(keys='datetime')
 
-            df = df[cols]
+                    df_list.append(df)
 
-            df = df.set_index(keys='datetime')
+            df = pd.concat(df_list)
             df = df.sort_index()
-            df = df[['open', 'high', 'low', 'close', 'volume']]
+
             return df
 
         else:
@@ -116,12 +165,21 @@ class Database:
             link_str = link_url + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + end_date_str
 
             response = requests.get(link_str)
+
             json_content = json.loads(response.content)
             json_content = json_content.replace(' nan', '\" nan\"')
 
-            result = ast.literal_eval(json_content)
+            result = json.loads(json_content.replace("'", "\""))
+
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
-            df = pd.DataFrame(result)
             df = df.sort_values(['date', 'market_cap_mil'], ascending=[True, False])
             df = df.reset_index(drop=True)
             return df
@@ -142,7 +200,7 @@ class Database:
         link_str = link_url + code_str + '&' + token_str + '&' + database_str
         
         response = requests.get(link_str)
-        result = ast.literal_eval(json.loads(response.content))
+        result = json.loads(json.loads(response.content).replace("'", "\""))
 
         df = pd.DataFrame(result)
         df['date'] = pd.to_datetime(df['date'], format = '%Y%m%d')
@@ -169,9 +227,17 @@ class Database:
             json_content = json.loads(response.content)
             json_content = json_content.replace(' nan', '\" nan\"')
 
-            result = ast.literal_eval(json_content)
+            #result = ast.literal_eval(json_content)
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             df = df.set_index(keys='date')
@@ -191,9 +257,10 @@ class Database:
         database_str = 'database=' + 'hk_earning_calendar_by_code'
         code_str = 'code=' + code
         link_str = link_url + code_str + '&' + token_str + '&' + database_str
-        print(link_str)
+
         response = requests.get(link_str)
-        result = ast.literal_eval(json.loads(response.content))
+        #result = ast.literal_eval(json.loads(response.content))
+        result = json.loads(json.loads(response.content).replace("'", "\""))
 
         df = pd.DataFrame(result)
         df['datetime'] = pd.to_datetime(df['datetime'], format = '%Y-%m-%d %H:%M:%S')
@@ -203,11 +270,9 @@ class Database:
         return df
     def get_hk_earning_calendar_by_date(self, start_date, end_date):
 
-        ### should change name to check_start_end_date ###
         check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            # link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'hk_earning_calendar_by_date'
@@ -218,7 +283,14 @@ class Database:
             response = requests.get(link_str)
             result = ast.literal_eval(json.loads(response.content))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
             df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d%H%M%S')
 
@@ -255,7 +327,6 @@ class Database:
         check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            #link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'us_earning_calendar_by_date'
@@ -264,9 +335,17 @@ class Database:
             link_str = link_url + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + end_date_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
+            df = df[['date', 'code']]
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
 
@@ -295,12 +374,20 @@ class Database:
             link_str = link_url + code_str + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + end_date_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
-            
-            df['date'] = df['date'].astype(str) 
-            df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                df['date'] = str(date_int)
+                df_list.append(df)
+
+            df = pd.concat(df_list)
+            df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
+            df = df.set_index('date')
+            df = df.sort_index()
+            df = df.reset_index(drop=False)
 
             return df
 
@@ -310,6 +397,7 @@ class Database:
                 if check_bool_dict[error] == False:
                     err_msg += error + ','
             print(err_msg)
+            
     def get_ccass_holding_rank(self, code, start_date, end_date):
 
         check_bool_dict = self.check_ccass_holding_rank_args(code, start_date, end_date)
@@ -324,9 +412,18 @@ class Database:
             link_str = link_url + code_str + '&' + token_str + '&' + database_str + '&' + start_date_str + '&' + end_date_str
 
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
+
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = date_int
+                    df_list.append(df)
+
+            df = pd.concat(df_list)
 
-            df = pd.DataFrame(result)
             df = df.sort_values(['date', 'share'], ascending=False)
             df = df[['date', 'ccass_id', 'name', 'share', 'percent']]
 
@@ -356,7 +453,6 @@ class Database:
         check_bool_dict = self.check_ccass_by_id_args(ccass_id, start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            # link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'ccass_by_id'
@@ -367,9 +463,17 @@ class Database:
                        end_date_str
 
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+
+            df = pd.concat(df_list)
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
 
@@ -409,9 +513,17 @@ class Database:
                        end_date_str
 
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df_first = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+
+            df_first = pd.concat(df_list)
 
             time.sleep(1)
 
@@ -421,9 +533,17 @@ class Database:
                        end_date_str
 
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df_last = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+
+            df_last = pd.concat(df_list)
 
             df_first = df_first.set_index(keys='code')
             df_last = df_last.set_index(keys='code')
@@ -477,7 +597,7 @@ class Database:
             
             response = requests.get(link_str)
             result = ast.literal_eval(json.loads(response.content))
-    
+
             df = pd.DataFrame(result)
             df['code'] = df['code'].str.zfill(5)
         
@@ -497,8 +617,8 @@ class Database:
             link_str = link_url + plate_name_str + '&' + token_str + '&' + database_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
-    
+            result = json.loads(json.loads(response.content).replace("'", "\""))
+
             df = pd.DataFrame(result)
             df['code'] = df['code'].str.zfill(5)
         
@@ -507,6 +627,8 @@ class Database:
         else:
             err_msg =  'index_name missing'
             print(err_msg)
+            
+        
     def get_all_hk_index_name(self):
 
         link_url = 'http://www.hkfdb.net/data_api?'
@@ -542,7 +664,7 @@ class Database:
         link_str = link_url + '&' + token_str + '&' + database_str
         
         response = requests.get(link_str)
-        result = ast.literal_eval(json.loads(response.content))
+        result = json.loads(json.loads(response.content).replace("'", "\""))
 
         df = pd.DataFrame(result)
         df['ipo_date'] = pd.to_datetime(df['ipo_date'],format='%Y-%m-%d')
@@ -571,9 +693,10 @@ class Database:
         json_content = json_content.replace(' nan', '\" nan\"')
 
         result = ast.literal_eval(json_content)
+
         df = pd.DataFrame(result)
         col_list = ['name','sponsors','accountants','valuers']
-        #col_list = ['name','sponsors','accountants']
+
         for col in col_list: 
             df[col] = df[col].str.replace('\n',' ', regex= False)
         for col in col_list:
@@ -591,7 +714,6 @@ class Database:
         check_bool_dict = self.check_market_highlight_args(market, start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            #link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'market_highlight'
@@ -602,9 +724,16 @@ class Database:
                        end_date_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
-
-            df = pd.DataFrame(result)
+            result = json.loads(json.loads(response.content).replace("'", "\""))
+            
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             df = df.drop_duplicates(subset='date',keep='last')
@@ -624,7 +753,6 @@ class Database:
         check_bool_dict = self.check_start_end_date(start_date, end_date)
 
         if False not in list(check_bool_dict.values()):
-            #link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             database_str = 'database=' + 'north_water'
@@ -634,9 +762,16 @@ class Database:
                        end_date_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
 
@@ -656,7 +791,6 @@ class Database:
         check_bool_dict = self.check_hk_deri_daily_market_report_args(deri, code ,start_date, end_date, exp)
 
         if False not in list(check_bool_dict.values()):
-            #link_url = 'http://localhost:8000/data_api?'
             link_url = 'http://www.hkfdb.net/data_api?'
             token_str = 'token=' + str(self.authToken)
             deri_str = 'deri=' + deri
@@ -670,9 +804,16 @@ class Database:
                        + '&' + deri_str  + '&' + code_str  + '&' + exp_str
             
             response = requests.get(link_str)
-            result = ast.literal_eval(json.loads(response.content))
+            result = json.loads(json.loads(response.content).replace("'", "\""))
 
-            df = pd.DataFrame(result)
+            df_list = []
+            for item in result:
+                date_int = item['_id']
+                df = pd.DataFrame(item['content'])
+                if len(df) > 0:
+                    df['date'] = str(date_int)
+                    df_list.append(df)
+            df = pd.concat(df_list)
 
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             
